use std::collections::HashMap;

use crate::termcolor::TermColor;

#[derive(Debug, Clone)]
pub struct CmdWorkerData {
    pub environment: HashMap<String, String>,
    pub command: String,
    pub args: Vec<String>,
}

#[derive(Debug, Clone)]
pub struct FuncWorkerOut {
    pub success: bool,
    pub out: Vec<String>,
}

impl Default for FuncWorkerOut {
    fn default() -> Self {
        Self {
            success: true,
            out: vec![],
        }
    }
}

pub struct FuncWorkerData {
    pub func: Box<dyn FnOnce() -> FuncWorkerOut + Send + Sync + 'static>,
}

pub enum WorkerJob {
    Cmd(CmdWorkerData),
    Func(FuncWorkerData),
}

impl WorkerJob {
    pub fn new_cmd(
        env: Option<HashMap<String, String>>,
        cmd: String,
        args: Option<Vec<String>>,
    ) -> Self {
        Self::Cmd(CmdWorkerData {
            environment: env.unwrap_or_default(),
            command: cmd,
            args: args.unwrap_or_default(),
        })
    }

    pub fn new_printer(txt: &str, color: Option<TermColor>) -> Self {
        let out = format!(
            "{}\n",
            if let Some(c) = color {
                c.colorize(txt)
            } else {
                txt.to_string()
            }
        );
        Self::new_func(Box::new(move || FuncWorkerOut {
            out: vec![out],
            success: true,
        }))
    }

    pub fn new_func(func: Box<dyn FnOnce() -> FuncWorkerOut + Send + Sync + 'static>) -> Self {
        Self::Func(FuncWorkerData { func })
    }
}
